#!/usr/bin/perl
# 00_local_spelling.t - Private test to spell check documentation
# $Id$

use warnings;
use strict;
use Test::More;

eval "use Test::Spelling";
plan skip_all => "Test::Spelling required for checking spelling"
    if $@;

set_spell_cmd('aspell list');
add_stopwords(<DATA>);
all_pod_files_spelling_ok();

__DATA__
5000BH
accelar
acl
actial
adsl
ADSL
af
agere
airespace
aironet
airos
alcatel
Alcatel
alteon
altiga
Altiga
AMAP
anycast
AOS
ap
APC
APs
arista
Arista
arn
arp
aruba
asa
asante
ascii
asn
ati
attr
ATUC
ATUR
augenstein
autonegotiate
autospecify
avaya
Avaya
baetz
Barthel
bayrs
baystack
bcn
begemot
Bernhard
bes
bgp
bigint
bigiron
Binks
bitmask
bladecenter
bln
bpdus
BSSID
bulkrepeaters
bulkwalk
Bulley
cabletron
casted
catos
cdp
centillion
CER
CES
circuitless
cisco
cisco's
ciscovtp
citrix
Citrix
cli
concentrators
conf
config
contivity
CPAN
CPE
cpu
ctron
CVTA
cyclades
DataSMART
datatype
De
deauthenticate
debugsnmp
dhcp
dlink
dmitry
DOCSIS
DRG
ds
dslam
dslams
dsss
dynamicgvrp
eapol
edgeiron
eigrp
enterasys
EOS
erszenyi
ess
ESXi
etherlike
ethernet
EvolutionOS
extranet
fastiron
fdb
fdp
fenner
fixups
fortinet
Fortinet
fortios
FreeBSD
FS
fsm
FTOS
fwsm
FWSM
gbe
gbesm
Gbps
getnext
Ghz
gigabit
Gorwits
hartmaier
huawei
Huawei
iana
ibm
icf
ieee
ietf
ifindex
ifStackTable
igmp
IgnoreNetSNMPConf
iid
iids
indices
ingen
InLoopback
ios
IOS
ip
IPv
IPV
ISID
jeroen
junos
Kentrox
Kosmach
kramarov
Kraus
ladvd
lans
lantronix
Lantronix
linksys
lldp
loopback
loopdetect
lorensen
lsb
lucent
Lucent
luiggi
lwapp
macfilter
mau
maxrepeaters
mbs
mckeown
mda
MERCHANTABILITY
mib
MIB
mibdirs
mibs
Mikrotik
milliwatts
MLT
MLX
MLXe
msb
mtu
multi
multicast
multimode
multiNode
multiseg
multivlan
munge
munges
namespace
ndp
netdisco
netgear
NetIron
netmask
netscaler
Netscaler
netscreen
Nic
Nicolai
nmm
nopassword
nortel
nosuch
NX
oem
ofdm
oid
oids
OmniSwitch
OO
os
osi
ospf
overridable
PacketFront
pae
PAgP
Palo
petri
pfSense
phy
pinkoski
poe
pov
powerconnect
PPI
procurve
propvirtual
proxim
PSE
pvid
PVST
qbridge
qos
queueing
Quidway
Rackswitch
radlan
radware
Radware
Ralf
rapidcity
readme
readonly
rebranded
retreved
retrynosuch
Robbert
rom
ros
RouterOS
rtt
rttmon
sca
Schenau
sergienko
SG
sieborger
Smartport
snmp
SNMP
SNMP's
snmpv2
snmpv3
snmpwalk
solaris
SonicWALL
sonmp
Sourceforge
SPBM
ssid
SSIDs
SSID's
SSL
stackable
stdout
steelhead
stg
Stickland
stp
subclassesand
subcomponents
subnet
sunos
supply's
synoptics
sys
Telesis
telesyn
telesys
terabit
tftp
tftpfilename
tftpserver
TiMOS
tmnxModel
todo
tuttle
Ubiquiti
ucsc
UDP
unicast
uninstall
untagged
uptime
vlan
vlans
VMware
VoIP
VPC
vpn
VRP
VSP
VSS
vtp
watanabe
wlan
wlans
WS
XMR
zoltan
zyxel
